@@ -11,7 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+import copy
 import itertools
 from math import ceil
 from typing import Dict, List, Optional, Union
@@ -19,18 +19,18 @@ from typing import Dict, List, Optional, Union
 import librosa
 import numpy as np
 import torch
-from omegaconf import DictConfig
+from hydra.utils import instantiate
+from omegaconf import DictConfig, OmegaConf, open_dict
 from pytorch_lightning import Trainer
+from torchmetrics import Accuracy
 from tqdm import tqdm
 
 from nemo.collections.asr.data.audio_to_label import AudioToSpeechLabelDataset
 from nemo.collections.asr.data.audio_to_label_dataset import get_tarred_speech_label_dataset
 from nemo.collections.asr.data.audio_to_text_dataset import convert_to_config_list
-from nemo.collections.asr.losses.angularloss import AngularSoftmaxLoss
 from nemo.collections.asr.models.asr_model import ExportableEncDecModel
 from nemo.collections.asr.parts.preprocessing.features import WaveformFeaturizer
 from nemo.collections.asr.parts.preprocessing.perturb import process_augmentations
-from nemo.collections.common.losses import CrossEntropyLoss as CELoss
 from nemo.collections.common.metrics import TopKClassificationAccuracy
 from nemo.collections.common.parts.preprocessing.collections import ASRSpeechLabel
 from nemo.core.classes import ModelPT
@@ -82,28 +82,91 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         )
         result.append(model)
 
+        model = PretrainedModelInfo(
+            pretrained_model_name="langid_ambernet",
+            location="https://api.ngc.nvidia.com/v2/models/nvidia/nemo/langid_ambernet/versions/1.12.0/files/ambernet.nemo",
+            description="For details about this model, please visit https://catalog.ngc.nvidia.com/orgs/nvidia/teams/nemo/models/langid_ambernet",
+        )
+        result.append(model)
+
         return result
 
     def __init__(self, cfg: DictConfig, trainer: Trainer = None):
         self.world_size = 1
+        self.cal_labels_occurrence_train = False
+        self.labels_occurrence = None
+
+        if 'num_classes' in cfg.decoder:
+            num_classes = cfg.decoder.num_classes
+        else:
+            num_classes = cfg.decoder.params.num_classes  # to pass test
+
+        if 'loss' in cfg:
+            if 'weight' in cfg.loss:
+                if cfg.loss.weight == 'auto':
+                    weight = num_classes * [1]
+                    self.cal_labels_occurrence_train = True
+                else:
+                    weight = cfg.loss.weight
+            else:
+                weight = None  # weight is None for angular loss and CE loss if it's not specified.
+
         if trainer is not None:
             self.world_size = trainer.num_nodes * trainer.num_devices
 
         super().__init__(cfg=cfg, trainer=trainer)
 
+        if self.labels_occurrence:
+            # Goal is to give more weight to the classes with less samples so as to match the ones with the higher frequencies
+            weight = [sum(self.labels_occurrence) / (len(self.labels_occurrence) * i) for i in self.labels_occurrence]
+
+        if 'loss' in cfg:
+            # To support older version checkpoints
+            if '_target_' not in cfg.loss:
+                logging.info(
+                    "Setting angular: true/false in decoder is deprecated and will be removed in 1.13 version, use specific loss with _target_"
+                )
+                OmegaConf.set_struct(cfg, True)
+                with open_dict(cfg):
+                    if 'angular' in cfg.decoder and cfg.decoder.angular:
+                        cfg.loss._target_ = "nemo.collections.asr.losses.angularloss.AngularSoftmaxLoss"
+                    else:
+                        # in case if specified angular=False but loss contained 'scale' or 'margin'
+                        cfg.loss.pop('scale', None)
+                        cfg.loss.pop('margin', None)
+                        cfg.loss._target_ = "nemo.collections.common.losses.cross_entropy.CrossEntropyLoss"
+
+            cfg_eval_loss = copy.deepcopy(cfg.loss)
+
+            if 'angular' in cfg.loss._target_:
+                OmegaConf.set_struct(cfg, True)
+                with open_dict(cfg):
+                    cfg.decoder.angular = True
+
+            if 'weight' in cfg.loss:
+                cfg.loss.weight = weight
+                cfg_eval_loss.weight = None
+
+            # May need a general check for arguments of loss
+            self.loss = instantiate(cfg.loss)
+            self.eval_loss = instantiate(cfg_eval_loss)
+
+        else:
+            tmp_loss_cfg = OmegaConf.create(
+                {"_target_": "nemo.collections.common.losses.cross_entropy.CrossEntropyLoss"}
+            )
+
+            self.loss = instantiate(tmp_loss_cfg)
+            self.eval_loss = instantiate(tmp_loss_cfg)
+
+        self._accuracy = TopKClassificationAccuracy(top_k=[1])
+
         self.preprocessor = EncDecSpeakerLabelModel.from_config_dict(cfg.preprocessor)
         self.encoder = EncDecSpeakerLabelModel.from_config_dict(cfg.encoder)
         self.decoder = EncDecSpeakerLabelModel.from_config_dict(cfg.decoder)
-        if 'angular' in cfg.decoder and cfg.decoder['angular']:
-            logging.info("loss is Angular Softmax")
-            scale = cfg.loss.scale
-            margin = cfg.loss.margin
-            self.loss = AngularSoftmaxLoss(scale=scale, margin=margin)
-        else:
-            logging.info("loss is Softmax-CrossEntropy")
-            self.loss = CELoss()
-        self.task = None
-        self._accuracy = TopKClassificationAccuracy(top_k=[1])
+
+        self._macro_accuracy = Accuracy(num_classes=num_classes, average='macro')
+
         self.labels = None
         if hasattr(self._cfg, 'spec_augment') and self._cfg.spec_augment is not None:
             self.spec_augmentation = EncDecSpeakerLabelModel.from_config_dict(self._cfg.spec_augment)
@@ -173,7 +236,10 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
                 min_duration=config.get('min_duration', None),
                 trim=config.get('trim_silence', False),
                 normalize_audio=config.get('normalize_audio', False),
+                cal_labels_occurrence=config.get('cal_labels_occurrence', False),
             )
+            if dataset.labels_occurrence:
+                self.labels_occurrence = dataset.labels_occurrence
 
         if hasattr(dataset, 'fixed_seq_collate_fn'):
             collate_fn = dataset.fixed_seq_collate_fn
@@ -192,6 +258,13 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         )
 
     def setup_training_data(self, train_data_layer_config: Optional[Union[DictConfig, Dict]]):
+        if self.cal_labels_occurrence_train:
+            # Calculate labels occurence for weighed CE loss for train set if weight equals 'auto'
+            # Note in this case, the cal_labels_occurrence in val_data_layer_config and test_data_layer_params need to be stay as False
+            OmegaConf.set_struct(train_data_layer_config, True)
+            with open_dict(train_data_layer_config):
+                train_data_layer_config['cal_labels_occurrence'] = True
+
         self.labels = self.extract_labels(train_data_layer_config)
         train_data_layer_config['labels'] = self.labels
         if 'shuffle' not in train_data_layer_config:
@@ -285,73 +358,63 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
 
         return {'loss': loss}
 
-    def validation_step(self, batch, batch_idx, dataloader_idx: int = 0):
+    def evaluation_step(self, batch, batch_idx, dataloader_idx: int = 0, tag: str = 'val'):
         audio_signal, audio_signal_len, labels, _ = batch
         logits, _ = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
-        loss_value = self.loss(logits=logits, labels=labels)
+        loss_value = self.eval_loss(logits=logits, labels=labels)
         acc_top_k = self._accuracy(logits=logits, labels=labels)
         correct_counts, total_counts = self._accuracy.correct_counts_k, self._accuracy.total_counts_k
+        self._macro_accuracy.update(preds=logits, target=labels)
+        stats = self._macro_accuracy._get_final_stats()
 
         return {
-            'val_loss': loss_value,
-            'val_correct_counts': correct_counts,
-            'val_total_counts': total_counts,
-            'val_acc_top_k': acc_top_k,
+            f'{tag}_loss': loss_value,
+            f'{tag}_correct_counts': correct_counts,
+            f'{tag}_total_counts': total_counts,
+            f'{tag}_acc_micro_top_k': acc_top_k,
+            f'{tag}_acc_macro_stats': stats,
         }
 
+    def multi_evaluation_epoch_end(self, outputs, dataloader_idx: int = 0, tag: str = 'val'):
+        loss_mean = torch.stack([x[f'{tag}_loss'] for x in outputs]).mean()
+        correct_counts = torch.stack([x[f'{tag}_correct_counts'] for x in outputs]).sum(axis=0)
+        total_counts = torch.stack([x[f'{tag}_total_counts'] for x in outputs]).sum(axis=0)
+
+        self._accuracy.correct_counts_k = correct_counts
+        self._accuracy.total_counts_k = total_counts
+        topk_scores = self._accuracy.compute()
+
+        self._macro_accuracy.tp = torch.stack([x[f'{tag}_acc_macro_stats'][0] for x in outputs]).sum(axis=0)
+        self._macro_accuracy.fp = torch.stack([x[f'{tag}_acc_macro_stats'][1] for x in outputs]).sum(axis=0)
+        self._macro_accuracy.tn = torch.stack([x[f'{tag}_acc_macro_stats'][2] for x in outputs]).sum(axis=0)
+        self._macro_accuracy.fn = torch.stack([x[f'{tag}_acc_macro_stats'][3] for x in outputs]).sum(axis=0)
+        macro_accuracy_score = self._macro_accuracy.compute()
+
+        self._accuracy.reset()
+        self._macro_accuracy.reset()
+
+        self.log(f'{tag}_loss', loss_mean, sync_dist=True)
+        for top_k, score in zip(self._accuracy.top_k, topk_scores):
+            self.log(f'{tag}_acc_micro_top@{top_k}', score, sync_dist=True)
+        self.log(f'{tag}_acc_macro', macro_accuracy_score, sync_dist=True)
+
+        return {
+            f'{tag}_loss': loss_mean,
+            f'{tag}_acc_micro_top_k': topk_scores,
+            f'{tag}_acc_macro': macro_accuracy_score,
+        }
+
+    def validation_step(self, batch, batch_idx, dataloader_idx: int = 0):
+        return self.evaluation_step(batch, batch_idx, dataloader_idx, 'val')
+
     def multi_validation_epoch_end(self, outputs, dataloader_idx: int = 0):
-        val_loss_mean = torch.stack([x['val_loss'] for x in outputs]).mean()
-        correct_counts = torch.stack([x['val_correct_counts'] for x in outputs]).sum(axis=0)
-        total_counts = torch.stack([x['val_total_counts'] for x in outputs]).sum(axis=0)
-
-        self._accuracy.correct_counts_k = correct_counts
-        self._accuracy.total_counts_k = total_counts
-        topk_scores = self._accuracy.compute()
-        self._accuracy.reset()
-
-        logging.info("val_loss: {:.3f}".format(val_loss_mean))
-        self.log('val_loss', val_loss_mean)
-        for top_k, score in zip(self._accuracy.top_k, topk_scores):
-            self.log('val_epoch_accuracy_top@{}'.format(top_k), score)
-
-        return {
-            'val_loss': val_loss_mean,
-            'val_acc_top_k': topk_scores,
-        }
+        return self.multi_evaluation_epoch_end(outputs, dataloader_idx, 'val')
 
     def test_step(self, batch, batch_idx, dataloader_idx: int = 0):
-        audio_signal, audio_signal_len, labels, _ = batch
-        logits, _ = self.forward(input_signal=audio_signal, input_signal_length=audio_signal_len)
-        loss_value = self.loss(logits=logits, labels=labels)
-        acc_top_k = self._accuracy(logits=logits, labels=labels)
-        correct_counts, total_counts = self._accuracy.correct_counts_k, self._accuracy.total_counts_k
-
-        return {
-            'test_loss': loss_value,
-            'test_correct_counts': correct_counts,
-            'test_total_counts': total_counts,
-            'test_acc_top_k': acc_top_k,
-        }
+        return self.evaluation_step(batch, batch_idx, dataloader_idx, 'test')
 
     def multi_test_epoch_end(self, outputs, dataloader_idx: int = 0):
-        test_loss_mean = torch.stack([x['test_loss'] for x in outputs]).mean()
-        correct_counts = torch.stack([x['test_correct_counts'] for x in outputs]).sum(axis=0)
-        total_counts = torch.stack([x['test_total_counts'] for x in outputs]).sum(axis=0)
-
-        self._accuracy.correct_counts_k = correct_counts
-        self._accuracy.total_counts_k = total_counts
-        topk_scores = self._accuracy.compute()
-        self._accuracy.reset()
-
-        logging.info("test_loss: {:.3f}".format(test_loss_mean))
-        self.log('test_loss', test_loss_mean)
-        for top_k, score in zip(self._accuracy.top_k, topk_scores):
-            self.log('test_epoch_accuracy_top@{}'.format(top_k), score)
-
-        return {
-            'test_loss': test_loss_mean,
-            'test_acc_top_k': topk_scores,
-        }
+        return self.multi_evaluation_epoch_end(outputs, dataloader_idx, 'test')
 
     @torch.no_grad()
     def get_embedding(self, path2audio_file):
@@ -362,7 +425,7 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
             path2audio_file: path to audio wav file
 
         Returns:
-            embs: speaker embeddings 
+            embs: speaker embeddings
         """
         audio, sr = librosa.load(path2audio_file, sr=None)
         target_sr = self._cfg.train_ds.get('sample_rate', 16000)
@@ -392,11 +455,11 @@ class EncDecSpeakerLabelModel(ModelPT, ExportableEncDecModel):
         Verify if two audio files are from the same speaker or not.
 
         Args:
-            path2audio_file1: path to audio wav file of speaker 1  
-            path2audio_file2: path to audio wav file of speaker 2 
+            path2audio_file1: path to audio wav file of speaker 1
+            path2audio_file2: path to audio wav file of speaker 2
             threshold: cosine similarity score used as a threshold to distinguish two embeddings (default = 0.7)
 
-        Returns:  
+        Returns:
             True if both audio files are from same speaker, False otherwise
         """
         embs1 = self.get_embedding(path2audio_file1).squeeze()
