@@ -306,13 +306,21 @@ class SpeechLabel(_Collection):
 class ASRSpeechLabel(SpeechLabel):
     """`SpeechLabel` collector from structured json files."""
 
-    def __init__(self, manifests_files: Union[str, List[str]], is_regression_task=False, *args, **kwargs):
+    def __init__(
+        self,
+        manifests_files: Union[str, List[str]],
+        is_regression_task=False,
+        cal_labels_occurrence=False,
+        *args,
+        **kwargs,
+    ):
         """Parse lists of audio files, durations and transcripts texts.
 
         Args:
             manifests_files: Either single string file or list of such -
                 manifests to yield items from.
-            is_regression_task: It's a regression task
+            is_regression_task: It's a regression task.
+            cal_labels_occurrence: whether to calculate occurence of labels.
             *args: Args to pass to `SpeechLabel` constructor.
             **kwargs: Kwargs to pass to `SpeechLabel` constructor.
         """
@@ -322,12 +330,16 @@ class ASRSpeechLabel(SpeechLabel):
             audio_files.append(item['audio_file'])
             durations.append(item['duration'])
             if not is_regression_task:
-                labels.append(item['label'])
+                label = item['label']
             else:
-                labels.append(float(item['label']))
+                label = float(item['label'])
 
+            labels.append(label)
             offsets.append(item['offset'])
 
+        if cal_labels_occurrence:
+            self.labels_occurrence = collections.Counter(labels)
+
         super().__init__(audio_files, durations, labels, offsets, *args, **kwargs)
 
     def __parse_item(self, line: str, manifest_file: str) -> Dict[str, Any]:
@@ -719,7 +731,7 @@ class DiarizationSpeechLabel(DiarizationLabel):
                 f"Manifest file has invalid json line " f"structure: {line} without proper audio file key."
             )
         item['audio_file'] = os.path.expanduser(item['audio_file'])
-        item['uniq_id'] = os.path.splitext(os.path.basename(item['rttm_filepath']))[0]
+        item['uniq_id'] = os.path.splitext(os.path.basename(item['audio_file']))[0]
         if 'duration' not in item:
             raise ValueError(f"Manifest file has invalid json line " f"structure: {line} without proper duration key.")
         item = dict(
