@@ -42,7 +42,7 @@ from nemo.collections.nlp.data.token_classification.punctuation_capitalization_d
     raise_not_equal_labels_error,
 )
 from nemo.collections.nlp.modules.common.tokenizer_utils import get_tokenizer
-from nemo.core.neural_types import ChannelType, LabelsType, MaskType, NeuralType
+from nemo.core.neural_types import AudioSignal, ChannelType, LabelsType, LengthsType, MaskType, NeuralType
 from nemo.utils import logging
 
 NUMBER_RE = "(0|[1-9][0-9]*)"
@@ -74,7 +74,7 @@ def count_lines_and_get_fragment_starting_positions(
 
     Args:
         file_name: a path to a text or label file
-        lines_per_dataset_fragment: number of lines in a dataset fragment. The last fragment can contain less lines
+        lines_per_dataset_fragment: number of lines in a dataset fragment. The last fragment can contain fewer lines
 
     Returns:
         num_lines: number of lines in a file
@@ -93,8 +93,8 @@ def count_lines_and_get_fragment_starting_positions(
 
 
 def get_fragment_start_bytes(
-    text_file: Path, labels_file: Path, lines_per_dataset_fragment: int
-) -> Tuple[int, List[int], List[int]]:
+    text_file: Path, labels_file: Path, lines_per_dataset_fragment: int, audio_file: Path = None
+) -> Union[Tuple[Any, Any, Any, Any], Tuple[Any, Any, Any]]:
     """
     A function for calculating borders of dataset fragments. The function is used to split ``text_file`` and
     ``labels_file`` for processing them in parallel.
@@ -103,6 +103,7 @@ def get_fragment_start_bytes(
         text_file: a path to a dataset source file
         labels_file: a path to a dataset label file
         lines_per_dataset_fragment: a number of lines in one fragment
+        audio_file: a path to a dataset audio file if one needed
 
     Returns:
         num_lines: total number of elements in the dataset (number of lines in ``text_file``` and ``labels_file``)
@@ -113,19 +114,34 @@ def get_fragment_start_bytes(
         f"Counting lines in files {text_file} and {labels_file} and creating segment borders. This may take "
         f"considerable time. 86GB, 1.27b lines file was processed in 7 minutes."
     )
-    result = Parallel(n_jobs=2)(
-        delayed(count_lines_and_get_fragment_starting_positions)(file_name, lines_per_dataset_fragment)
-        for file_name in [text_file, labels_file]
-    )
-    if result[0][0] != result[1][0]:
-        raise ValueError(
-            f"Text file {text_file} and label file {labels_file} contain different number of lines. Number of lines "
-            f"in text file: {result[0][0]}, number of lines in label file: {result[1][0]}."
+    if audio_file:
+        result = Parallel(n_jobs=3)(
+            delayed(count_lines_and_get_fragment_starting_positions)(file_name, lines_per_dataset_fragment)
+            for file_name in [text_file, labels_file, audio_file]
         )
-    num_lines = result[0][0]
-    text_start_bytes, label_start_bytes = result[0][1], result[1][1]
-    assert len(text_start_bytes) == len(label_start_bytes)
-    return num_lines, text_start_bytes, label_start_bytes
+        num_lines = result[0][0]
+        if result[0][0] != result[1][0]:
+            raise ValueError(
+                f"Text file {text_file} and label file {labels_file} contain different number of lines. Number of lines "
+                f"in text file: {result[0][0]}, number of lines in label file: {result[1][0]}."
+            )
+        text_start_bytes, label_start_bytes, manifest_start_bytes = result[0][1], result[1][1], result[2][1]
+        assert len(text_start_bytes) == len(label_start_bytes) == len(manifest_start_bytes)
+        return num_lines, text_start_bytes, label_start_bytes, manifest_start_bytes
+    else:
+        result = Parallel(n_jobs=2)(
+            delayed(count_lines_and_get_fragment_starting_positions)(file_name, lines_per_dataset_fragment)
+            for file_name in [text_file, labels_file]
+        )
+        num_lines = result[0][0]
+        if result[0][0] != result[1][0]:
+            raise ValueError(
+                f"Text file {text_file} and label file {labels_file} contain different number of lines. Number of lines "
+                f"in text file: {result[0][0]}, number of lines in label file: {result[1][0]}."
+            )
+        text_start_bytes, label_start_bytes = result[0][1], result[1][1]
+        assert len(text_start_bytes) == len(label_start_bytes)
+        return num_lines, text_start_bytes, label_start_bytes
 
 
 def process_fragment(
@@ -152,6 +168,10 @@ def process_fragment(
     batch_mark_up_progress_queue: mp.Queue,
     batch_building_progress_queue: mp.Queue,
     writing_to_tar_progress_queue: mp.Queue,
+    audio_file: Path = None,
+    sample_rate: int = None,
+    audio_file_start_pos: int = None,
+    use_audio: bool = False,
 ) -> None:
     tokenizer = get_tokenizer(
         tokenizer_name,
@@ -163,12 +183,21 @@ def process_fragment(
     )
     tmp_text: Optional[str] = None
     tmp_labels: Optional[str] = None
+    tmp_audio: Optional[str] = None
     try:
         otfd, tmp_text = tempfile.mkstemp(suffix='.txt', prefix=f'text_{fragment_idx}_', dir=output_dir, text=True)
         olfd, tmp_labels = tempfile.mkstemp(suffix='.txt', prefix=f'labels_{fragment_idx}_', dir=output_dir, text=True)
+        if use_audio:
+            oafd, tmp_audio = tempfile.mkstemp(
+                suffix='.txt', prefix=f'audio_{fragment_idx}_', dir=output_dir, text=True
+            )
         with text_file.open() as tf, labels_file.open() as lf, os.fdopen(otfd, 'w') as otf, os.fdopen(
             olfd, 'w'
-        ) as olf:
+        ) as olf:  # handle audio manifest
+            if use_audio:
+                mf = audio_file.open()
+                mf.seek(audio_file_start_pos)
+                oaf = os.fdopen(oafd, 'w')
             tf.seek(text_start_pos)
             lf.seek(label_start_pos)
             for _ in range(lines_per_dataset_fragment):
@@ -177,6 +206,11 @@ def process_fragment(
                     break
                 otf.write(text_line)
                 olf.write(lf.readline())
+                if use_audio:
+                    oaf.write(mf.readline())
+        if use_audio:
+            mf.close()
+            oaf.close()
         dataset = BertPunctuationCapitalizationDataset(
             tmp_text,
             tmp_labels,
@@ -193,12 +227,19 @@ def process_fragment(
             tokenization_progress_queue=tokenization_progress_queue,
             batch_mark_up_progress_queue=batch_mark_up_progress_queue,
             batch_building_progress_queue=batch_building_progress_queue,
+            audio_file=tmp_audio,
+            sample_rate=sample_rate,
+            use_audio=use_audio,
+            use_bucketing=True,
+            preload_audios=use_audio,
         )
     finally:
         if tmp_text is not None and os.path.exists(tmp_text):
             os.remove(tmp_text)
         if tmp_labels is not None and os.path.exists(tmp_labels):
             os.remove(tmp_labels)
+        if tmp_audio is not None and os.path.exists(tmp_audio):
+            os.remove(tmp_audio)
     dataset.features_pkl.unlink()
     tar_ctr = 0
     current_file_name = output_dir / TAR_FRAGMENT_TMPL_IN_PROGRESS.format(fragment_idx=fragment_idx, file_idx=tar_ctr)
@@ -352,7 +393,7 @@ def create_label_dictionaries(
         labels_file: a path to file with labels
         text_start_bytes: indices of first bytes of fragments in ``labels_file``
         num_lines: total number of lines in ``labels_file``
-        lines_per_dataset_fragment: number of lines in dataset fragments. The last fragment can have less lines
+        lines_per_dataset_fragment: number of lines in dataset fragments. The last fragment can have fewer lines
         pad_label: a label used for padding and for absence of punctuation and capitalization
         n_jobs: a number of fragments processed in parallel
 
@@ -678,6 +719,9 @@ def create_tarred_dataset(
     capit_label_vocab_file: Optional[Union[os.PathLike, str]] = None,
     tar_file_prefix: Optional[str] = 'punctuation_capitalization',
     n_jobs: Optional[int] = None,
+    audio_file: Optional[Path] = None,
+    use_audio: Optional[bool] = False,
+    sample_rate: Optional[int] = 16000,
 ) -> None:
     """
     Creates tarred dataset from ``text_file`` and ``labels_file``. A tarred dataset allows to train on large amounts of
@@ -704,7 +748,7 @@ def create_tarred_dataset(
     instance of the class will handle iteration and constructing masks and token types for BERT model.
 
     Args:
-        text_file (:obj:`Union[os.PathLike, str]`): a path to a file with dataset source. Dataset source is lowercased
+        text_file (:obj:`Union[os.PathLike, str]`): a path to a file with dataset source. Dataset source is lowercase
             text without punctuation. Number of lines in ``text_file`` has to be equal to the number of lines in
             ``labels_file``.
         labels_file (:obj:`Union[os.PathLike, str]`): a path to a file with labels. Labels are given in the format
@@ -723,7 +767,7 @@ def create_tarred_dataset(
             before packing them into batches. Reducing ``lines_per_dataset_fragment`` leads to reducing of the amount
             of memory used by this function.
         num_batches_per_tarfile (:obj:`int`): a number of batches saved in a tar file. If you increase
-            ``num_batches_per_tarfile``, then there will be less tar files in the dataset. There cannot be less then
+            ``num_batches_per_tarfile``, then there will be less tar files in the dataset. There cannot be less than
             ``num_batches_per_tarfile`` batches in a tar file, and all excess batches are removed. Maximum number of
             discarded batches is ``num_batches_per_tarfile - 1``.
         tokenizer_name (:obj:`str`): a name of the tokenizer used for tokenization of source sequences. Possible
@@ -760,6 +804,9 @@ def create_tarred_dataset(
             file names start. The string can contain only characters ``A-Z``, ``a-z``, ``0-9``, ``_``, ``-``, ``.``.
         n_jobs (:obj:`int`, `optional`): a number of workers for creating tarred dataset. If ``None``, then ``n_jobs``
             is equal to number of CPUs.
+        audio_file (:obj:`Optional[Union[os.PathLike, str]]`, defaults to :obj:`None`): a path to a file with audio dataset file paths if dataset is lexical and audio. Must contain one path per line.
+        use_audio (:obj:`bool`, `optional`, defaults to :obj:`False`): If set to ``True`` dataset becomes lexical and audio rather than only lexical.
+        sample_rate (:obj:`int`, `optional`, defaults to :obj:`16000`) Targeted sample rate of audios If ``use_audio`` set to ``True``.
     """
     check_tar_file_prefix(tar_file_prefix, ValueError, 'tar_file_prefix')
     if n_jobs is None:
@@ -775,9 +822,15 @@ def create_tarred_dataset(
     output_file_tmpl = ds_params_str + TAR_FINAL_TMPL
     metadata_file_name = output_dir / ('metadata.' + ds_params_str + '.json')
     remove_unexpected_files_and_dirs(output_dir, output_file_tmpl, metadata_file_name)
-    num_lines, text_start_bytes, label_start_bytes = get_fragment_start_bytes(
-        text_file, labels_file, lines_per_dataset_fragment
-    )
+    audio_start_bytes = None
+    if use_audio:
+        num_lines, text_start_bytes, label_start_bytes, audio_start_bytes = get_fragment_start_bytes(
+            text_file, labels_file, lines_per_dataset_fragment, audio_file
+        )
+    else:
+        num_lines, text_start_bytes, label_start_bytes = get_fragment_start_bytes(
+            text_file, labels_file, lines_per_dataset_fragment
+        )
     if text_start_bytes:
         output_dir.mkdir(parents=True, exist_ok=True)
     else:
@@ -820,8 +873,18 @@ def create_tarred_dataset(
                 capit_label_ids,
                 fragment_idx,
                 *progress_queues,
+                audio_file,
+                sample_rate,
+                audio_file_start_pos,
+                use_audio,
+            )
+            for fragment_idx, (text_start_pos, label_start_pos, audio_file_start_pos) in enumerate(
+                zip(
+                    text_start_bytes,
+                    label_start_bytes,
+                    audio_start_bytes if use_audio else [None for _ in range(len(text_start_bytes))],
+                )
             )
-            for fragment_idx, (text_start_pos, label_start_pos) in enumerate(zip(text_start_bytes, label_start_bytes))
         )
     repack_tar_files_with_not_enough_batches(output_dir, num_batches_per_tarfile)
     create_metadata_file(output_dir, output_file_tmpl, metadata_file_name, num_batches_per_tarfile)
@@ -845,7 +908,7 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
             ``'punct_label_vocab_file'``, ``'capit_label_vocab_file'`` items. The first item is total number of batches
             in a dataset, the second is a list of paths to tar files relative to directory containing
             ``metadata_file``. Items ``'punct_label_vocab_file'`` and ``'capit_label_vocab_file'`` are paths to
-            ``.csv`` files which contain unique punctuation an capitalization label vocabularies. Vocabulary file paths
+            ``.csv`` files which contain unique punctuation a capitalization label vocabularies. Vocabulary file paths
             are relative to directory containing the ``metadata_file``. Each line in ``'punct_label_vocab_file'`` and
             ``'capit_label_vocab_file'`` contains 1 label. The first lines in ``'punct_label_vocab_file'`` and
             ``'capit_label_vocab_file'`` files are neutral labels which also serve as pad labels. Neutral labels for
@@ -875,13 +938,13 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
             a str value during ddp.
             -   ``'scatter'``: The default shard strategy applied by WebDataset, where each node gets
                 a unique set of shards, which are permanently pre-allocated and never changed at runtime.
-            -   ``'replicate'``: Optional shard strategy, where each node gets all of the set of shards
+            -   ``'replicate'``: Optional shard strategy, where each node gets all the set of shards
                 available in the tarred dataset, which are permanently pre-allocated and never changed at runtime.
                 The benefit of replication is that it allows each node to sample data points from the entire
                 dataset independently of other nodes, and reduces dependence on value of :param:`shuffle_n`.
 
                 .. warning::
-                    Replicated strategy allows every node to sample the entire set of available tarfiles,
+                    Replicated strategy allows every node to sample the entire set of available tar files,
                     and therefore more than one node may sample the same tarfile, and even sample the same
                     data points! As such, there is no assured guarantee that all samples in the dataset will be
                     sampled at least once during 1 epoch. Scattered strategy, on the other hand, on specific
@@ -892,7 +955,19 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
 
     @property
     def output_types(self) -> Optional[Dict[str, NeuralType]]:
-        """Returns neural types of batches yielded by this dataset."""
+        """Returns definitions of module output ports. """
+        if self.use_audio:
+            return {
+                'input_ids': NeuralType(('B', 'T'), ChannelType()),
+                'segment_ids': NeuralType(('B', 'T'), ChannelType()),
+                'input_mask': NeuralType(('B', 'T'), MaskType()),
+                'subtokens_mask': NeuralType(('B', 'T'), MaskType()),
+                'loss_mask': NeuralType(('B', 'T'), MaskType()),
+                'punct_labels': NeuralType(('B', 'T'), LabelsType()),
+                'capit_labels': NeuralType(('B', 'T'), LabelsType()),
+                'features': NeuralType(('B', 'T'), AudioSignal()),
+                'features_length': NeuralType(('B', 'T'), LengthsType()),
+            }
         return {
             'input_ids': NeuralType(('B', 'T'), ChannelType()),
             'segment_ids': NeuralType(('B', 'T'), ChannelType()),
@@ -915,6 +990,7 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
         global_rank: int = 0,
         shuffle_n: int = 1,
         shard_strategy: str = "scatter",
+        use_audio: bool = False,
     ) -> None:
         super().__init__()
 
@@ -983,6 +1059,8 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
             logging.info("WebDataset will not shuffle files within the tar files.")
         self._dataset = self._dataset.to_tuple('__key__', 'batch.pyd').map(f=self._build_sample)
 
+        self.use_audio = use_audio
+
     def _check_pad_label(self) -> None:
         """
         Checks the condition that ``pad_label`` passed to this class constructor has ``0`` id in
@@ -1021,7 +1099,7 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
                 :class:`~nemo.collections.nlp.models.token_classification.punctuation_capitalization_model.PunctuationCapitalizationModel`
                 in which this tarred dataset is used.
             class_labels: a config item ``model.class_labels``. See more in description of
-                :ref:`class labels config<class-labels-config-label>`.
+                :ref:`class labels' config<class-labels-config-label>`.
             common_dataset_parameters_config: a config item ``model.common_dataset_parameters``. See more in
                 of :ref:`common dataset parameters config<common-dataset-parameters-config-label>`.
         """
@@ -1177,8 +1255,7 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
     def __len__(self) -> int:
         return self.length
 
-    @staticmethod
-    def collate_fn(batches: List[Dict[str, np.ndarray]]) -> Dict[str, torch.Tensor]:
+    def collate_fn(self, batches: List[Dict[str, np.ndarray]]) -> Dict[str, torch.Tensor]:
         """
         Return zeroth batch of ``batches`` list passed for collating and casts ``'segment_ids'``, ``'punct_labels'``,
         ``'capit_labels'`` to types supported by
@@ -1207,4 +1284,6 @@ class BertPunctuationCapitalizationTarredDataset(IterableDataset):
         batch['segment_ids'] = batch['segment_ids'].int()
         batch['punct_labels'] = batch['punct_labels'].long()
         batch['capit_labels'] = batch['capit_labels'].long()
+        if self.use_audio:
+            batch['features'] = batch['features'].to(torch.float32)
         return batch
