@@ -71,10 +71,6 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
 
     def __init__(self, cfg: DictConfig, trainer: Trainer):
         super().__init__(cfg, trainer=trainer)
-        if cfg.get('pipeline_model_parallel_size', 1) > 2 and self.cfg.get('position_embedding_type') == 'relative':
-            raise ValueError(
-                "pipeline_model_parallel_size cannot be > 2 with position_embedding_type == relative at the moment."
-            )
         if cfg.get('pipeline_model_parallel_size', 1) > 1:
             if cfg.get('pipeline_model_parallel_split_rank', 0) <= 0:
                 raise ValueError(
@@ -369,14 +365,13 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
     def allreduce_word_and_position_embeddings(self):
 
         # Modified from megatron-lm: https://github.com/NVIDIA/Megatron-LM/blob/d41696840ed0a7edb7e0499eb82a48ae112d9bb3/megatron/training.py#L407
-        # All-reduce word_embeddings' grad across first and last stages to ensure
-        # that word_embeddings parameters stay in sync.
-        # This should only run for models that support pipelined model parallelism
-        # (BERT and GPT-2).
+        # All-reduce word_embeddings' grad across first, last stages to ensure that word_embeddings parameters stay in sync.
         if parallel_state.get_pipeline_model_parallel_world_size() > 1 and (
             parallel_state.is_rank_in_embedding_group()
         ):
-            if self.enc_dec_model.share_token_embeddings:
+            if self.cfg.get('share_token_embeddings', True) and self.cfg.get(
+                'share_decoder_tokens_head_embeddings', True
+            ):
                 word_embeddings_weight = self.enc_dec_model.word_embeddings_weight()
                 if self.megatron_amp_o2:
                     # O2 recipe stores a "main" copy of weights and grads
@@ -384,20 +379,74 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
                 else:
                     grad = word_embeddings_weight.grad
                 torch.distributed.all_reduce(grad, group=parallel_state.get_embedding_group())
+            else:
+                raise ValueError(
+                    f"Attempting to allreduce word_embeddings for pipeline parallel size > 1, but found untied word embeddings or token head embeddings. This is not supported yet."
+                )
 
-                # All reduce position embeddings for T5.
-                if (
-                    parallel_state.is_rank_in_position_embedding_group()
-                    and parallel_state.get_pipeline_model_parallel_world_size() > 1
-                    and parallel_state.get_pipeline_model_parallel_split_rank() is not None
-                    and self.cfg.get('position_embedding_type') == 'learned_absolute'
-                ):
-                    position_embeddings_weight = self.enc_dec_model.position_embeddings_weight()
+        # All-reduce position embeddings for T5.
+        if (
+            parallel_state.is_rank_in_position_embedding_group()
+            and parallel_state.get_pipeline_model_parallel_world_size() > 1
+            and parallel_state.get_pipeline_model_parallel_split_rank() is not None
+            and self.cfg.encoder.get('position_embedding_type') == 'learned_absolute'
+            and self.cfg.decoder.get('position_embedding_type') == 'learned_absolute'
+        ):
+            if self.cfg.get('share_token_embeddings', True):
+                position_embeddings_weight = self.enc_dec_model.position_embeddings_weight()
+                if self.megatron_amp_o2:
+                    grad = position_embeddings_weight.main_grad
+                else:
+                    grad = position_embeddings_weight.grad
+                torch.distributed.all_reduce(grad, group=parallel_state.get_position_embedding_group())
+
+        # All-reduce relative position embeddings for T5.
+        if (
+            parallel_state.get_pipeline_model_parallel_world_size()
+            > 2  # This > 2 and not > 1 since with PP=2 encoder RPE can live only on one rank.
+            and parallel_state.get_pipeline_model_parallel_split_rank() is not None
+        ):
+            # For split rank = 1, we have only one encoder rank and so we don't need to allreduce.
+            if (
+                self.cfg.encoder.get('position_embedding_type') == 'relative'
+                and parallel_state.is_rank_in_encoder_relative_position_embedding_group()
+                and parallel_state.get_pipeline_model_parallel_split_rank() > 1
+            ):
+                position_embeddings_weight = self.enc_dec_model.encoder_relative_position_embeddings_weight()
+                if self.megatron_amp_o2:
+                    grad = position_embeddings_weight.main_grad
+                else:
+                    grad = position_embeddings_weight.grad
+                torch.distributed.all_reduce(
+                    grad, group=parallel_state.get_encoder_relative_position_embedding_group()
+                )
+
+            # For split rank == pipeline_world_size - 1, we have only one decoder rank and so we don't need to allreduce.
+            if (
+                self.cfg.decoder.get('position_embedding_type') == 'relative'
+                and parallel_state.is_rank_in_decoder_relative_position_embedding_group()
+            ):
+                position_embeddings_weight = self.enc_dec_model.decoder_relative_position_embeddings_weight()
+                if self.megatron_amp_o2:
+                    grad = position_embeddings_weight.main_grad
+                else:
+                    grad = position_embeddings_weight.grad
+                torch.distributed.all_reduce(
+                    grad, group=parallel_state.get_decoder_relative_position_embedding_group()
+                )
+
+                # If the model also has separate RPE weights for decoder cross-attention, allreduce those as well.
+                if not self.cfg.decoder.get('relative_position_bias_self_attention_only', True):
+                    position_embeddings_weight = (
+                        self.enc_dec_model.decoder_cross_attention_relative_position_embeddings_weight()
+                    )
                     if self.megatron_amp_o2:
                         grad = position_embeddings_weight.main_grad
                     else:
                         grad = position_embeddings_weight.grad
-                    torch.distributed.all_reduce(grad, group=parallel_state.get_position_embedding_group())
+                    torch.distributed.all_reduce(
+                        grad, group=parallel_state.get_decoder_relative_position_embedding_group()
+                    )
 
     def get_forward_output_and_loss_func(self):
         def fwd_output_and_loss_func(batch, model):
@@ -431,7 +480,8 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
         Computed on first call, and then cached.
         """
         # build mapping of kwargs to arg index at first run
-        args_name = inspect.getfullargspec(self.enc_dec_model.forward)[0][1:]
+        module = self.enc_dec_model.forward if not self.megatron_amp_o2 else self.enc_dec_model.module.forward
+        args_name = inspect.getfullargspec(module)[0][1:]
         kwargs_to_arg_idx = {k: v for k, v in zip(args_name, range(len(args_name)))}
 
         return kwargs_to_arg_idx
@@ -594,8 +644,6 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
         return loss_mean
 
     def validation_epoch_end(self, outputs):
-        if not outputs:
-            return
         if parallel_state.is_pipeline_last_stage():
             # only the last pipeline parallel stages return loss
             averaged_loss = torch.stack(outputs).mean()
@@ -618,8 +666,6 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
         return self.validation_step(batch, batch_idx)
 
     def test_epoch_end(self, outputs):
-        if not outputs:
-            return
         if parallel_state.is_pipeline_last_stage():
             # only the last pipeline parallel stages return loss
             averaged_loss = torch.stack(outputs).mean()
@@ -806,9 +852,28 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
 
         # when using pipeline model parallel the final stage need to initialize word embeddings
         if parallel_state.get_pipeline_model_parallel_world_size() > 1:
+            assert (
+                self.cfg.share_token_embeddings
+            ), "share_word_embedding must be True when using pipeline model parallel > 1"
+            assert (
+                self.cfg.share_decoder_tokens_head_embeddings
+            ), "share_decoder_tokens_head_embeddings must be True when using pipeline model parallel > 1"
             self.enc_dec_model.sync_initial_word_embeddings()
-            if self.cfg.get('position_embedding_type') != 'relative':
+            if (
+                self.cfg.encoder.get('position_embedding_type') != 'relative'
+                and self.cfg.decoder.get('position_embedding_type') != 'relative'
+            ):
                 self.enc_dec_model.sync_initial_position_embeddings()
+            # Synchronize RPE embeddings across pipeline parallel ranks.
+            else:
+                if self.cfg.encoder.get('position_embedding_type', 'learned_absolute') == 'relative':
+                    self.enc_dec_model.sync_initial_encoder_relative_position_embeddings()
+                if self.cfg.decoder.get('position_embedding_type', 'learned_absolute') == 'relative':
+                    self.enc_dec_model.sync_initial_decoder_relative_position_embeddings()
+                if self.cfg.decoder.get(
+                    'position_embedding_type', 'learned_absolute'
+                ) == 'relative' and not self.cfg.decoder.get('relative_position_bias_self_attention_only', True):
+                    self.enc_dec_model.sync_initial_decoder_cross_attention_relative_position_embeddings()
 
     def setup_training_data(self, cfg):
         if hasattr(self, '_train_ds'):
@@ -914,6 +979,8 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
         forward_step_func = self._get_forward_output_only_func(
             arg_names=arg_names, output_name="hiddens", output_enc_hidden_only=True
         )
+
+        # Counter intuitively, we need to set decoder_sequence_length=encoder_seq_length because while running `.enocde()`, the last hidden states from encoder are passed through as identity through the pipeline. Setting it to anything else will cause hanging due to tensor shape mismatches.
         if self.cfg.get('pipeline_model_parallel_size', 1) > 1:
             output_tensor = forward_backward_pipelining_without_interleaving(
                 forward_step_func=forward_step_func,
@@ -921,7 +988,7 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
                 model=self.enc_dec_model,
                 forward_only=True,
                 tensor_shape=tensor_shape,
-                decoder_sequence_length=1,
+                decoder_sequence_length=encoder_seq_length,
                 dtype=self.autocast_dtype,
             )
         else:
@@ -931,12 +998,11 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
                 model=self.enc_dec_model,
                 forward_only=True,
                 tensor_shape=tensor_shape,
-                decoder_sequence_length=1,
+                decoder_sequence_length=encoder_seq_length,
                 dtype=self.autocast_dtype,
             )
 
-        # get output tensor of encoder [seq_len, batch, hidden]
-        if parallel_state.is_pipeline_last_stage():
+        if output_tensor:
             output_tensor = output_tensor[0]['hiddens']
         else:
             output_tensor = torch.zeros(tensor_shape, dtype=self.autocast_dtype).cuda()
@@ -1022,6 +1088,7 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
         # get encoder hiddens (output)
         if enc_output is None:
             # Encode returns a tensr of shape [batch, seq_len, hidden]
+            # All ranks will call `.encode()`, but only the last rank will have a non-empty output tensor.
             enc_output = self.encode(
                 tokens_enc=tokens_enc, enc_mask=enc_mask, encoder_input=encoder_input, reconfigure_microbatch=False
             )
@@ -1069,6 +1136,7 @@ class MegatronLMEncoderDecoderModel(MegatronBaseModel):
                         dim=-1, index=torch.tensor(ignore_ids, device=device), value=-float('Inf')
                     )
 
+                # TODO: do log_softmax in fp32?
                 log_probs, token_ids = torch.max(torch.nn.functional.log_softmax(output_tensor, dim=-1), dim=-1)
                 predicted_tokens_dec = torch.cat(
                     [predicted_tokens_dec.to(token_ids.device), token_ids[:, -1].unsqueeze(1)], dim=1
