@@ -63,6 +63,7 @@ def get_language_model(
     fp32_residual_connection=False,
     activations_checkpoint_method=None,
     activations_checkpoint_num_layers=1,
+    normalization='layernorm',
     layernorm_epsilon=1e-5,
     bias_gelu_fusion=True,
     masked_softmax_fusion=True,
@@ -113,6 +114,7 @@ def get_language_model(
         fp32_residual_connection=fp32_residual_connection,
         activations_checkpoint_method=activations_checkpoint_method,
         activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+        normalization=normalization,
         layernorm_epsilon=layernorm_epsilon,
         bias_gelu_fusion=bias_gelu_fusion,
         masked_softmax_fusion=masked_softmax_fusion,
@@ -387,6 +389,7 @@ class TransformerLanguageModel(MegatronModule):
         fp32_residual_connection=False,
         activations_checkpoint_method=None,
         activations_checkpoint_num_layers=1,
+        normalization='layernorm',
         layernorm_epsilon=1e-5,
         bias_gelu_fusion=True,
         masked_softmax_fusion=True,
@@ -454,6 +457,7 @@ class TransformerLanguageModel(MegatronModule):
             fp32_residual_connection=fp32_residual_connection,
             activations_checkpoint_method=activations_checkpoint_method,
             activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+            normalization=normalization,
             layernorm_epsilon=layernorm_epsilon,
             hidden_dropout=hidden_dropout,
             use_cpu_initialization=use_cpu_initialization,
@@ -488,6 +492,7 @@ class TransformerLanguageModel(MegatronModule):
                 fp32_residual_connection=fp32_residual_connection,
                 activations_checkpoint_method=activations_checkpoint_method,
                 activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+                normalization=normalization,
                 layernorm_epsilon=layernorm_epsilon,
                 hidden_dropout=hidden_dropout,
                 use_cpu_initialization=use_cpu_initialization,
