@@ -60,6 +60,7 @@ def get_encoder_model(
     fp32_residual_connection=False,
     activations_checkpoint_method=None,
     activations_checkpoint_num_layers=1,
+    activations_checkpoint_granularity=None,
     layernorm_epsilon=1e-5,
     bias_activation_fusion=True,
     bias_dropout_add_fusion=True,
@@ -78,6 +79,8 @@ def get_encoder_model(
     chunk_size=64,
     num_self_attention_per_cross_attention=1,
     layer_number_offset=0,  # this is use only for attention norm_factor scaling
+    sequence_parallel=False,
+    gradient_accumulation_fusion=False,
 ):
     """Build language model and return along with the key to save."""
 
@@ -148,6 +151,7 @@ def get_encoder_model(
             fp32_residual_connection=fp32_residual_connection,
             activations_checkpoint_method=activations_checkpoint_method,
             activations_checkpoint_num_layers=activations_checkpoint_num_layers,
+            activations_checkpoint_granularity=activations_checkpoint_granularity,
             layernorm_epsilon=layernorm_epsilon,
             bias_activation_fusion=bias_activation_fusion,
             bias_dropout_add_fusion=bias_dropout_add_fusion,
@@ -162,6 +166,8 @@ def get_encoder_model(
             parent_model_type=parent_model_type,
             chunk_size=chunk_size,
             layer_number_offset=layer_number_offset,
+            sequence_parallel=sequence_parallel,
+            gradient_accumulation_fusion=gradient_accumulation_fusion,
         )
     elif arch == "perceiver":
         encoder = MegatronPerceiverEncoderModule(
