@@ -15,7 +15,7 @@
 """Utilities for models."""
 
 import math
-from typing import Dict, List, Union
+from typing import Dict, List, Tuple, Union
 
 import torch
 
@@ -25,7 +25,7 @@ try:
     from apex.transformer.enums import AttnMaskType
     from apex.transformer.pipeline_parallel.schedules.common import listify_model
     from apex.transformer.tensor_parallel.layers import linear_with_grad_accumulation_and_async_allreduce
-    from apex.contrib.layer_norm.layer_norm import FastLayerNorm
+    from apex.transformer.layers.layer_norm import FastLayerNorm
 
     HAVE_APEX = True
 except (ImportError, ModuleNotFoundError):
@@ -106,6 +106,13 @@ def init_method_normal(sigma):
     return init_
 
 
+def init_method_const(val):
+    def init_(tensor):
+        return torch.nn.init.constant_(tensor, val)
+
+    return init_
+
+
 def scaled_init_method_normal(sigma, num_layers):
     """Init method based on N(0, sigma/sqrt(2*num_layers)."""
     std = sigma / math.sqrt(2.0 * num_layers)
@@ -331,3 +338,16 @@ def get_params_for_weight_decay_optimization(
                 )
 
     return weight_decay_params, no_weight_decay_params
+
+
+def get_all_params_for_weight_decay_optimization(
+    model: Union[torch.nn.Module, List[torch.nn.Module]],
+) -> Tuple[Dict[str, List[torch.nn.Parameter]]]:
+    """Use all params for weight decay."""
+    modules = listify_model(model)
+
+    weight_decay_params = [
+        p for module in modules for module_ in module.modules() for p in module_._parameters.values() if p is not None
+    ]
+
+    return ({'params': weight_decay_params},)
