@@ -68,6 +68,7 @@ class TokenizerConfig:
     coverage: Optional[float] = 0.999
     training_sample_size: Optional[int] = None
     r2l: Optional[bool] = False
+    sentencepiece_legacy: Optional[bool] = False
 
 
 def get_tokenizer(
@@ -124,7 +125,7 @@ def get_tokenizer(
     elif tokenizer_name == 'char':
         return CharTokenizer(vocab_file=vocab_file, **special_tokens_dict)
     elif tokenizer_name == 'regex':
-        return RegExTokenizer().load_tokenizer(tokenizer_model)
+        return RegExTokenizer().load_tokenizer(regex_file=tokenizer_model, vocab_file=vocab_file)
 
     logging.info(
         f"Getting HuggingFace AutoTokenizer with pretrained_model_name: {tokenizer_name}, vocab_file: {vocab_file}, merges_files: {merges_file}, "
@@ -196,7 +197,7 @@ def get_nmt_tokenizer(
         return ByteLevelTokenizer(special_tokens_dict)
     elif library == 'regex':
         logging.info(f'Using regex tokenization')
-        return RegExTokenizer().load_tokenizer(tokenizer_model)
+        return RegExTokenizer().load_tokenizer(regex_file=tokenizer_model, vocab_file=vocab_file)
     elif library == 'megatron':
         if model_name in megatron_tokenizer_model_map:
             model_name = megatron_tokenizer_model_map[model_name]
