@@ -46,7 +46,7 @@ from nemo.utils import logging, model_utils
 
 @dataclass
 class G2PConfig:
-    _target_: str = "nemo.collections.tts.torch.g2ps.EnglishG2p"
+    _target_: str = "nemo_text_processing.g2p.modules.EnglishG2p"
     phoneme_dict: str = "scripts/tts_dataset_files/cmudict-0.7b_nv22.08"
     heteronyms: str = "scripts/tts_dataset_files/heteronyms-052722"
     phoneme_probability: float = 0.5
@@ -54,7 +54,7 @@ class G2PConfig:
 
 @dataclass
 class TextTokenizer:
-    _target_: str = "nemo.collections.tts.torch.tts_tokenizers.EnglishPhonemesTokenizer"
+    _target_: str = "nemo.collections.common.tokenizers.text_to_speech.tts_tokenizers.EnglishPhonemesTokenizer"
     punct: bool = True
     stresses: bool = True
     chars: bool = True
@@ -181,6 +181,16 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
 
     def _setup_tokenizer(self, cfg):
         text_tokenizer_kwargs = {}
+
+        if "phoneme_dict" in cfg.text_tokenizer:
+            text_tokenizer_kwargs["phoneme_dict"] = self.register_artifact(
+                "text_tokenizer.phoneme_dict", cfg.text_tokenizer.phoneme_dict,
+            )
+        if "heteronyms" in cfg.text_tokenizer:
+            text_tokenizer_kwargs["heteronyms"] = self.register_artifact(
+                "text_tokenizer.heteronyms", cfg.text_tokenizer.heteronyms,
+            )
+
         if "g2p" in cfg.text_tokenizer:
             g2p_kwargs = {}
 
@@ -436,10 +446,10 @@ class FastPitchModel(SpectrogramGenerator, Exportable):
         mel_loss = collect("mel_loss")
         dur_loss = collect("dur_loss")
         pitch_loss = collect("pitch_loss")
-        self.log("v_loss", val_loss)
-        self.log("v_mel_loss", mel_loss)
-        self.log("v_dur_loss", dur_loss)
-        self.log("v_pitch_loss", pitch_loss)
+        self.log("val_loss", val_loss)
+        self.log("val_mel_loss", mel_loss)
+        self.log("val_dur_loss", dur_loss)
+        self.log("val_pitch_loss", pitch_loss)
 
         _, _, _, _, spec_target, spec_predict = outputs[0].values()
 
