@@ -114,6 +114,16 @@ class NoamAnnealingParams(WarmupSchedulerParams):
     min_lr: float = 0.0
 
 
+@dataclass
+class NoamHoldAnnealingParams(WarmupHoldSchedulerParams):
+    """
+    Polynomial Hold Decay Annealing parameter config
+    It is not derived from Config as it is not a NeMo object (and in particular it doesn't need a name).
+    """
+
+    decay_rate: float = 0.5
+
+
 @dataclass
 class WarmupAnnealingParams(WarmupSchedulerParams):
     """
@@ -270,6 +280,7 @@ AVAILABLE_SCHEDULER_PARAMS = {
     'SquareRootConstantSchedulerParams': SquareRootConstantSchedulerParams,
     'CosineAnnealingParams': CosineAnnealingParams,
     'NoamAnnealingParams': NoamAnnealingParams,
+    'NoamHoldAnnealingParams': NoamHoldAnnealingParams,
     'WarmupAnnealingParams': WarmupAnnealingParams,
     'PolynomialDecayAnnealingParams': PolynomialDecayAnnealingParams,
     'PolynomialHoldDecayAnnealingParams': PolynomialHoldDecayAnnealingParams,
