@@ -14,7 +14,7 @@
 
 import copy
 from functools import partial
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, Optional, Union
 
 import hydra
 import torch
