@@ -17,6 +17,7 @@ import re
 import subprocess
 import sys
 import time
+import warnings
 from copy import deepcopy
 from dataclasses import dataclass
 from datetime import timedelta
@@ -24,6 +25,7 @@ from pathlib import Path
 from shutil import copy, move
 from typing import Any, Dict, List, Optional, Union
 
+import pytorch_lightning
 import torch
 from hydra.core.hydra_config import HydraConfig
 from hydra.utils import get_original_cwd
@@ -32,6 +34,7 @@ from pytorch_lightning.callbacks import Callback, ModelCheckpoint
 from pytorch_lightning.callbacks.timer import Interval, Timer
 from pytorch_lightning.loggers import LoggerCollection as _LoggerCollection
 from pytorch_lightning.loggers import TensorBoardLogger, WandbLogger
+from pytorch_lightning.loops import TrainingEpochLoop
 from pytorch_lightning.strategies.ddp import DDPStrategy
 
 from nemo.constants import NEMO_ENV_VARNAME_TESTING, NEMO_ENV_VARNAME_VERSION
@@ -119,6 +122,8 @@ class ExpManagerConfig:
     # Configures creation of log files for different ranks
     log_local_rank_0_only: Optional[bool] = False
     log_global_rank_0_only: Optional[bool] = False
+    # disable initial validation when resuming from a checkpoint saved during validation
+    disable_validation_on_resume: Optional[bool] = True
 
 
 class TimingCallback(Callback):
@@ -332,6 +337,10 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
             trainer, log_dir, checkpoint_name, cfg.resume_if_exists, cfg.checkpoint_callback_params
         )
 
+    if cfg.disable_validation_on_resume:
+        # extend training loop to skip initial validation when resuming from checkpoint
+        configure_no_restart_validation_training_loop(trainer)
+
     if is_global_rank_zero():
         # Move files_to_copy to folder and add git information if present
         if cfg.files_to_copy:
@@ -810,19 +819,30 @@ class NeMoModelCheckpoint(ModelCheckpoint):
         if trainer.fast_dev_run:
             return None
 
+        # check if we need to save a last checkpoint manually as validation isn't always run based on the interval
+        if self.save_last and trainer.val_check_interval != 0:
+            should_save_last_checkpoint = False
+            if isinstance(trainer.val_check_interval, float) and trainer.val_check_interval % trainer.global_step != 0:
+                should_save_last_checkpoint = True
+            if isinstance(trainer.val_check_interval, int) and trainer.global_step % trainer.val_check_interval != 0:
+                should_save_last_checkpoint = True
+            if should_save_last_checkpoint:
+                monitor_candidates = self._monitor_candidates(trainer)
+                super()._save_last_checkpoint(trainer, monitor_candidates)
         # Call parent on_train_end() to save the -last checkpoint
         super().on_train_end(trainer, pl_module)
 
         # Load the best model and then re-save it
         if self.save_best_model:
             # wait for all processes
-            trainer.training_type_plugin.barrier("SaveBestCheckpointConnector.resume_end")
+            trainer.strategy.barrier("SaveBestCheckpointConnector.resume_end")
             if self.best_model_path == "":
                 logging.warning(
                     f"{self} was told to save the best checkpoint at the end of training, but no saved checkpoints "
                     "were found. Saving latest model instead."
                 )
             else:
+                self.best_model_path = trainer.strategy.broadcast(self.best_model_path)
                 trainer._checkpoint_connector.restore(self.best_model_path)
 
         if self.save_nemo_on_train_end:
@@ -928,3 +948,25 @@ class StatelessTimer(Timer):
 
     def load_state_dict(self, state_dict: Dict[str, Any]) -> None:
         return
+
+
+def configure_no_restart_validation_training_loop(trainer: pytorch_lightning.Trainer) -> None:
+    if type(trainer.fit_loop.epoch_loop) != TrainingEpochLoop:
+        warnings.warn("Detected custom epoch loop. Skipping no validation on restart support.", UserWarning)
+        return
+    loop = SkipResumeTrainingValidationLoop(trainer.min_steps, trainer.max_steps)
+    loop.trainer = trainer
+    trainer.fit_loop.epoch_loop = loop
+
+
+class SkipResumeTrainingValidationLoop(TrainingEpochLoop):
+    """
+    Extend the PTL Epoch loop to skip validating when resuming.
+    This happens when resuming a checkpoint that has already run validation, but loading restores
+    the training state before validation has run.
+    """
+
+    def _should_check_val_fx(self) -> bool:
+        if self.restarting and self.global_step % self.trainer.val_check_batch == 0:
+            return False
+        return super()._should_check_val_fx()
