@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import pynini
 from nemo_text_processing.inverse_text_normalization.vi.graph_utils import (
     NEMO_CHAR,
     NEMO_DIGIT,
@@ -20,14 +21,7 @@ from nemo_text_processing.inverse_text_normalization.vi.graph_utils import (
     delete_space,
     insert_space,
 )
-
-try:
-    import pynini
-    from pynini.lib import pynutil
-
-    PYNINI_AVAILABLE = True
-except (ModuleNotFoundError, ImportError):
-    PYNINI_AVAILABLE = False
+from pynini.lib import pynutil
 
 
 class TimeFst(GraphFst):
