@@ -14,20 +14,14 @@
 # limitations under the License.
 
 
+import pynini
 from nemo_text_processing.inverse_text_normalization.vi.graph_utils import (
     NEMO_CHAR,
     NEMO_SIGMA,
     GraphFst,
     delete_space,
 )
-
-try:
-    import pynini
-    from pynini.lib import pynutil
-
-    PYNINI_AVAILABLE = True
-except (ModuleNotFoundError, ImportError):
-    PYNINI_AVAILABLE = False
+from pynini.lib import pynutil
 
 
 class WhiteListFst(GraphFst):
